import { Request, Response, CookieOptions, NextFunction } from 'express';
import { HttpError } from 'http-errors';

type TokenRetriever = (req: Request) => string | null | undefined;
type DoubleCsrfCookieOptions = Omit<CookieOptions, "httpOnly">;
declare module "http" {
    interface IncomingHttpHeaders {
        "x-csrf-token"?: string | undefined;
    }
}
declare module "express-serve-static-core" {
    interface Request {
        csrfToken?: (overwrite?: boolean) => ReturnType<CsrfTokenCreator>;
        ignoreCsrfToken?: boolean;
    }
}
type CsrfSecretRetriever = (req?: Request) => string | Array<string>;
type DoubleCsrfConfigOptions = Partial<DoubleCsrfConfig> & {
    getSecret: CsrfSecretRetriever;
};
type doubleCsrfProtection = (req: Request, res: Response, next: NextFunction) => void;
type RequestMethod = "GET" | "HEAD" | "PATCH" | "PUT" | "POST" | "DELETE" | "CONNECT" | "OPTIONS" | "TRACE";
type CsrfIgnoredMethods = Array<RequestMethod>;
type CsrfRequestValidator = (req: Request) => boolean;
type CsrfTokenCreator = (req: Request, res: Response, ovewrite?: boolean, validateOnReuse?: boolean) => string;
interface DoubleCsrfConfig {
    getSecret: CsrfSecretRetriever;
    cookieName: string;
    size: number;
    cookieOptions: DoubleCsrfCookieOptions;
    ignoredMethods: CsrfIgnoredMethods;
    getTokenFromRequest: TokenRetriever;
}
interface DoubleCsrfUtilities {
    invalidCsrfTokenError: HttpError;
    generateToken: CsrfTokenCreator;
    validateRequest: CsrfRequestValidator;
    doubleCsrfProtection: doubleCsrfProtection;
}

declare function doubleCsrf({ getSecret, cookieName, cookieOptions: { sameSite, path, secure, ...remainingCookieOptions }, size, ignoredMethods, getTokenFromRequest, }: DoubleCsrfConfigOptions): DoubleCsrfUtilities;

export { doubleCsrf };
